<?php
/*

Plugin Name: Galette User Sync (GUS)
Plugin URI: http://colas.sebastien.free.fr/projets/gus/
Description: This plugin allow you to synchronize WordPress users into Galette.
Version: 13.05
Author: Sebastien Colas 
License: GPL2


	Copyright 2013  Colas Sébastien  (email : colas.sebastien@free.fr)

    This program is free software; you can redistribute it and/or modify
    it under the terms of the GNU General Public License, version 2, as 
    published by the Free Software Foundation.

    This program is distributed in the hope that it will be useful,
    but WITHOUT ANY WARRANTY; without even the implied warranty of
    MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
    GNU General Public License for more details.

    You should have received a copy of the GNU General Public License
    along with this program; if not, write to the Free Software
    Foundation, Inc., 51 Franklin St, Fifth Floor, Boston, MA  02110-1301  USA

*/

/*
 * This method update a user profile in the galette user table
 * arg1 => hashmap containing the configuration
 * arg2 => user to synchronize in galette
 */
function update_user_in_galette($configuration,$user)
{
	// We use wpdb variable to make SQL request in database
	global $wpdb;
	
	// Creation of a hashmap 'data' to update the database
	// If the field in wordpress is set we push the data in Galette
	// data variable will contain something like that
	// data['pseudo_adh']='login_wordpress'
	foreach ($configuration['field_mapping'] as $field_from => $field_to)
		if (isset($user->$field_from) && ($user->$field_from!=''))
			$data[$field_to]=$user->$field_from;
	// Here we add the modification date
	$data['date_modif_adh']=date('Y-m-d');
	
	// We update the galette database with update
	// reminder:
	// arg1 => database table name (usually galette_adherents)
	// arg2 => data to update in the table
	// arg3 => where close usualy pseudo_adh='login_wordpress'
	$wpdb->update( $configuration['table_name'] , $data, 
	array( $configuration['field_mapping'][$configuration['field_principal']] => $user->$configuration['field_principal'] ));
}

/*
 * This method create a user in the galette user table
 * arg1 => hashmap containing the configuration
 * arg2 => user to synchronize in galette
 */
function create_user_in_galette($configuration,$user)
{
	// We use wpdb variable to make SQL request in database	
	global $wpdb;

	// Creation of a hashmap 'data' to create user in the database
	// We push the wordpress value of a field in the corresponding field in Galette
	// data variable will contain something like that
	// data['pseudo_adh']='login_wordpress'
	foreach ($configuration['field_mapping'] as $field_from => $field_to)
		$data[$field_to]=$user->$field_from;

	// Here we add the creation date
	$data['date_crea_adh']=date('Y-m-d');
	// Here we add the modification date	
	$data['date_modif_adh']=date('Y-m-d');
	// Here we set the galette user to 'active member'	
	$data['activite_adh']=1;

	// We insert the user in galette user table
	// reminder:
	// arg1 => database table name (usually galette_adherents)
	// arg2 => data to insert in the table
	$wpdb->insert($configuration['table_name'],$data);
}

/*
 * This function will wynchronise Wordpress user informations with galette user informations
 * arg1 => hashmap containing the configuration
 * arg2 => List of the user to synchronize
 */
function synchronize($configuration,$gus_users)
{
	// We use wpdb variable to make SQL request in database	
	global $wpdb;
	// for each user to synchronize
	foreach ($gus_users->results as $user)
	{	
		// How many users in Galette has the principal field name corresponding to WordPress user
		$sql="SELECT COUNT(*) FROM ".$configuration['table_name']." WHERE ".$configuration['field_mapping'][$configuration['field_principal']]."='".$user->$configuration['field_principal']."'";
		$user_count = $wpdb->get_var( $sql );
		
		switch ($user_count)
		{
			// 0 : user does not exist in galette so create it and update the synchonization date in meta WordPress
			case 0:		create_user_in_galette($configuration,$user);
						update_user_meta( $user->ID, 'gus_sync_date', date('Y-m-d H:i'));
						break;
			// 1 : user exist in galette so update the fields and update the synchonization date in meta WordPress						
			case 1:		update_user_in_galette($configuration,$user);
						update_user_meta( $user->ID, 'gus_sync_date', date('Y-m-d H:i'));
						break;
			// Other cases are not yet supported so nothing to do
		}
	}
}

/*
 * This function add a new column in the WordPress user list
 */
function gus_add_custom_field( $defaults ) 
{
	// Here we set the name of the new column
	$defaults['gus_sync'] = __('GUS', 'gus-plugin');
	return $defaults;
}

/*
 * This function show the value of the new column in the WordPress user list
 */
function gus_custom_field_value($value, $column_name, $id) 
{
	// If it is the new plugin column
	if( $column_name == 'gus_sync' ) 
	{
		// If the logged user can edit WordPress users
		if ( current_user_can('edit_users') )
		{
			// Get the if we have to synchronize the user to galette
			$gus_sync=get_user_meta($id, 'gus_sync',true);
			// If not set show No else show the value
			if ($gus_sync=='') $gus_sync='No';
			return __($gus_sync);
		}
	}
}

/*
 * Main function of the plugin the admin page
 */
function gus_admin_page()
{
?>
<div class="wrap">
<div id="icon-tools" class="icon32"><br /></div><h2>Galette User Sync (GUS)</h2>
<?		

	global $wpdb;
	$site_url=get_option('siteurl');
	$current_url=$_SERVER['REQUEST_URI'];
	// Get the configuration of the plugin
	$configuration=get_option('GUS_configuration');
	
	// Get the current action
	$action=$_POST['action'];

	switch ($action)
	{
		// Set the galette installation path
		case 'set_gal_path':
							// Put the installation path of galette in the configuration
							$configuration['galette_path']=$_POST['galette_path'];
							// Save the configuration in WordPress option
							update_option( 'GUS_configuration', $configuration );	
							break;
		// Enable the SSO with Galette
		case 'sso_on':		// Put the SSO flag to on in the configuration
							$configuration['SSO_wp_gal']=true;
							// Save the configuration in WordPress option
							update_option( 'GUS_configuration', $configuration );
							break;
		// Disable the SSO with Galette
		case 'sso_off':	// Remove the SSO flag to on in the configuration
							unset($configuration['SSO_wp_gal']);
							// Save the configuration in WordPress option
							update_option( 'GUS_configuration', $configuration );
							break;
		// Launch the synchronization
		case 'sync':		// Get the list of the users to synchronize
							$gus_users = new WP_User_Query( array( 'meta_key' => 'gus_sync', 'meta_value' => 'Yes') );
							// Synchronize the users		
							synchronize($configuration,$gus_users);
							break;
		// Remove users from users to synchronize
		case 'delete': 	// Remove the meta flag from WordPress to disable the user synchronization
							if (is_array($_POST['list']))
								foreach ($_POST['list'] as $user_id)
									delete_user_meta( $user_id, 'gus_sync');
 							break;
 		// Add users to users to synchronize
		case 'add':		// Set the WordPress meta flag to enable users synchronization
							if (is_array($_POST['list']))
								foreach ($_POST['list'] as $user_id)
									add_user_meta( $user_id, 'gus_sync', 'Yes',true );
							break;
		// Set the Galette table name
		case 'db_config':	if (isset($_POST['table_name']))
							{
								// Put the table name in the configuration variable
								$configuration['table_name']=$_POST['table_name'];
								// Save the configuration in WordPress option
								update_option( 'GUS_configuration', $configuration );
							}
							break;
		// Add a field mapping in the configuration
		case 'add_field':	if (($_POST['field_name']!='') && ($_POST['field_value']!=''))
							{
								// If the source field exist replace the mapping else set the mapping
								if (isset ($configuration['field_mapping']))
										$configuration['field_mapping'][$_POST['field_name']]=$_POST['field_value'];  
								else 	
										$configuration['field_mapping']=array($_POST['field_name'] => $_POST['field_value']);
								// Save the configuration in WordPress option
								update_option( 'GUS_configuration', $configuration );										
							}							
							break;
		// Delete a field in the configuration
		case 'del_field':	// Remove the mapping from the configuration variable
							unset($configuration['field_mapping'][$_POST['field_name']]);
							// If it is the principal field remove it							
							if ($configuration['field_principal']==$_POST['field_name']) unset($configuration['field_principal']);
							// Save the configuration in WordPress option							
							update_option( 'GUS_configuration', $configuration );								
							break;
		// Set the default configuration
		case 'reset':		// Put the default configuration in the configuration variable
							$configuration=array('table_name' => 'galette_adherents',
												'field_principal' => 'user_login',
												'field_mapping' => array(
		                                    		'user_login'		=>'pseudo_adh',
		                                    		'last_name' 		=> 'nom_adh',
		                                    		'first_name' 		=> 'prenom_adh',
		                                    		'user_email' 		=> 'email_adh',
		                                    		'street_address' 	=> 'adresse_adh',
		                                    		'postal_code' 		=> 'cp_adh',
		                                    		'locality' 		=> 'ville_adh',
		                                    		'phone_mobile' 	=> 'gsm_adh',
		                                    		'phone_home' 		=> 'tel_adh',
		                                    		'user_nicename' 	=> 'login_adh'
		                                    	));
							// Save the configuration in WordPress option
							update_option( 'GUS_configuration', $configuration );
							break;
		// Set the principal field to make correspondance between WordPress and Galette
		case 'principal':	// Set the name of the principal field
							$configuration['field_principal']=$_POST['field_principal'];
							// Save the configuration in WordPress option		
							update_option( 'GUS_configuration', $configuration );
							break;
	}	
	
	if (isset($_GET['config']))
	{
		// Show the configuration menu
?>
<form method='post'>
<input type='hidden' name='action' value='db_config'>
<h3><? _e('DataBase Configuration','gus-plugin') ?></h3>
<? _e('Gallette Member Table Name :','gus-plugin')?> <input type='text' name='table_name' value='<? echo $configuration['table_name']?>'>
<? $table_name=$configuration['table_name'];
if($wpdb->get_var("SHOW TABLES LIKE '$table_name'") != $table_name)
 echo '<b>'.__('Error','gus-plugin').'</b>'; else _e('OK','gus-plugin'); ?><br>
<input type='submit' value='<? _e('Apply','gus-plugin') ?>'>
</form>
<h3><? _e('Fields Mapping','gus-plugin') ?></h3>

<table border="1">
<tr>
  <th><? _e('WordPress User Field or Meta','gus-plugin') ?></th><th><? _e('Galette User Field','gus-plugin')?></th><th><? _e('Action','gus-plugin') ?></th>
</tr>

<? 
	if (isset ($configuration['field_mapping']))
	foreach ($configuration['field_mapping'] as $key => $value)
	{
?>		
		<form method='post'>
		<input type="hidden" name="action" value="del_field">
		<input type="hidden" name="field_name" value="<? echo $key ?>">
		<tr><td><input type="text" size="30" value="<? echo $key ?>" disabled></td><td><input type="text" size="30" value="<? echo $value ?>" disabled></td><td><input type="submit" value="<? _e('Remove','gus-plugin') ?>"></td></tr>
		</form>
<?
	}
?>
<form method='post'>
<input type='hidden' name='action' value='add_field'>
<tr><td><input type='text' size='30' name='field_name'></td><td><input type='text' size='30' name='field_value'></td><td><input type='submit' value='<? _e('Add','gus-plugin') ?>'></td></tr>
</form>
</table>
<h3><? _e('Principal Field','gus-plugin') ?></h3>
<form method='post'>
<input type='hidden' name='action' value='principal'>
<select name='field_principal'>
<? 
	if (!isset($configuration['field_principal'])) echo '<option>'.__('Please set !!!','gus-plugin').'</option>';
	if (isset($configuration['field_mapping']))
	foreach ($configuration['field_mapping'] as $key => $value)
	{
		if ($configuration['field_principal']==$key)
		echo '<option selected="selected" value='.$key.'>'.$key.'</option>';
		else
		echo '<option value='.$key.'>'.$key.'</option>';
  	} 
?>
</select>
<input type='submit' value='<? _e('Apply','gus-plugin') ?>'>
</form>

<h3><? _e('Default configuration','gus-plugin') ?></h3>
<form method='post'>
<input type='hidden' name='action' value='reset'>
<input type='submit' value='<? _e('Reset to default configuration','gus-plugin') ?>'>
</form>

<h3><? _e('Wordpress User List to Sync with Galette','gus-plugin') ?></h3>
<a href='<? echo str_replace('&config','',$current_url) ?>'><? _e('Return to Users to sync with GUS','gus-plugin') ?></a>

<h3><? _e('SSO WordPress to Galette','gus-plugin') ?></h3>
<?		if (!function_exists('runkit_function_redefine')) _e('Runkit must be installed to activate SSO<br><a href="http://us3.php.net/manual/en/runkit.installation.php" target="_new">http://us3.php.net/manual/en/runkit.installation.php</a>','gus-plugin');
		else
		{
?>
<form method='post'>
<input type='hidden' name='action' value='set_gal_path'>
<? _e('Galette Installation Path : ','gus-plugin') ?><input type='text' name='galette_path' value='<? echo $configuration['galette_path'] ?>'><input type='submit' value='<? _e('Set Path','gus-plugin') ?>'>
<br><? _e('(Galette installation path is relative to WordPress installation path)','gus-plugin') ?>
</form>
<br>
<?			
			$filename='../'.$configuration['galette_path'].'/includes/galette.inc.php';
			if (file_exists($filename))
			if ($configuration['SSO_wp_gal'])
			{
				_e('SSO is ENABLED !!!','gus-plugin');
?>
<form method='post'>
<input type='hidden' name='action' value='sso_off'>
<input type='submit' value='<? _e('Disable SSO','gus-plugin') ?>'>
</form>
<?				
			}
			else
			{
				_e('SSO is DISABLED !!!','gus-plugin');				
?>
<form method='post'>
<input type='hidden' name='action' value='sso_on'>
<input type='submit' value='<? _e('Enable SSO','gus-plugin') ?>'>
</form>
<?
			}
			else _e('Incorrect Path !!!','gus-plugin');
		}
	}
	else
	{	
		// Show the user mapping menu
		if (!isset($gus_users))
			$gus_users = new WP_User_Query( array( 'meta_key' => 'gus_sync', 'meta_value' => 'Yes') );
		$non_gust_users = new WP_User_Query( array('meta_key'=>'gus_sync','meta_compare' => 'NOT EXISTS') );
		$all_users = get_users();	
		
		$gus_users_count=count($gus_users->results);
		$users_count=count($all_users);
		$non_gust_users_count=count($non_gust_users->results);
		
		
		if (isset($_REQUEST['users'])) $user_filter=$_REQUEST['users'];
		else $user_filter='gus';
		
		switch ($user_filter)
		{
			case 'all':	$list=$all_users;break;
			case 'gus':	$list=$gus_users->results;break;
			case 'others':	$list=$non_gust_users->results;break;
			default:		$list=null;break;
		}
		
		if (!$configuration) echo '<h3><a href="'.$current_url.'&config">'.__('Please Configure GUS','gus-plugin').'</a></h3>';
		else 
		{
			echo '<p><a href="'.$current_url.'&config">'.__('Access to GUS Configuration','gus-plugin').'</a></p>';
		
		if ($user_filter=='gus')
		{
?>
<form method='post'><input type='hidden' name='action' value='sync'><input type='submit' value='<? _e('Synchronize NOW !!!','gus-plugin')?>'></form>

<? } } ?></p>

<ul class='subsubsub'>
	<li><a href='tools.php?page=galette_user_sync.php&users=all'<? echo $user_filter=='all'?' class="current"':'' ?>><? _e('All Users','gus-plugin')?> <span class="count">(<? echo $users_count ?>)</span></a> |</li>
	<li><a href='tools.php?page=galette_user_sync.php'<? echo $user_filter=='gus'?' class="current"':'' ?>><? _e('Users sync with GUS','gus-plugin')?><span class="count">(<? echo $gus_users_count ?>)</span></a> |</li>
	<li><a href='tools.php?page=galette_user_sync.php&users=others'<? echo $user_filter=='others'?' class="current"':'' ?>><? _e('Other users','gus-plugin') ?> <span class="count">(<? echo $non_gust_users_count ?>)</span></a></li>
</ul>

<form method="post" action="<? echo $current_url ?>">

<table class="wp-list-table widefat fixed users" cellspacing="0">
	<thead>
	<tr>
		<th scope='col' id='cb' class='manage-column column-cb check-column'>
		<label class="screen-reader-text" for="cb-select-all-1">Tout sélectionner</label>
		<input id="cb-select-all-1" type="checkbox" />
		</th>
		
		<th scope='col' id='username'>
		<? _e('Username','gus-plugin') ?>
		</th>
		
		<th scope='col' id='last_name'>
		<? _e('Last Name','gus-plugin') ?>
		</th>
		
		<th scope='col' id='first_name'>
		<? _e('First Name','gus-plugin') ?>
		</th>
				
		<th scope='col' id='email'>
		<? _e('E-mail','gus-plugin') ?>
		</th>
				
		<th scope='col' id='gus_sync'>
		<? _e('Galette User Sync','gus-plugin')?>
		</th>

		<th scope='col' id='gus_last_sync'>
		<? _e('Last Sync','gus-plugin')?>
		</th>
	</tr>		
	</thead>
	
	<tfoot>
	<tr>
		<th scope='col'  class='manage-column column-cb check-column'  style="">
		<label class="screen-reader-text" for="cb-select-all-2">Tout sélectionner</label>
		<input id="cb-select-all-2" type="checkbox" />
		</th>
		
		<th scope='col' id='username'>
		<? _e('Username','gus-plugin') ?>
		</th>
		
		<th scope='col' id='last_name'>
		<? _e('Last Name','gus-plugin') ?>
		</th>
		
		<th scope='col' id='first_name'>
		<? _e('First Name','gus-plugin') ?>
		</th>
				
		<th scope='col' id='email'>
		<? _e('E-mail','gus-plugin') ?>
		</th>
				
		<th scope='col' id='gus_sync'>
		<? _e('Galette User Sync','gus-plugin')?>
		</th>

		<th scope='col' id='gus_last_sync'>
		<? _e('Last Sync','gus-plugin')?>
		</th>
	</tr>
	</tfoot>	
	
	<tbody id="the-list" data-wp-lists='list:user'>
<? foreach ($list as $user) { ?>
		<tr id='user-1' class="alternate">
		  <th scope='row' class='check-column'><label class="screen-reader-text">Sélectionner <? echo $user->user_login ?></label><input type='checkbox' name='list[]' value='<? echo $user->ID ?>' /></th>
		  <td class="username column-username"><strong><a href="<? echo $site_url?>/wp-admin/user-edit.php?user_id=<? echo $user->ID ?>"><? echo $user->user_login ?></a></strong></td>
		  <td><? echo $user->last_name ?></td>
		  <td><? echo $user->first_name ?></td>
		  <td><a href='mailto:<? echo $user->user_email ?>' ><? echo $user->user_email ?></a></td>
		  <td><? _e($user->gus_sync); ?></td>
		  <td><? echo $user->gus_sync_date ?></td></tr>
<? } ?>		  
	</tbody>
</table>	
	<div class="tablenav bottom">
		<div class="alignleft actions">
			<select name='action'>
				<option value='-1' selected='selected'><? _e('Bulk Actions','gus-plugin')?></option>
				<option value='add'><? _e('Synchronize with GUS','gus-plugin') ?></option>
				<option value='delete'><? _e('Remove GUS Synchonization','gus-plugin') ?></option>				
			</select>
			<input type="submit" name="" class="button action" value="<? _e('Apply','gus-plugin') ?>"  />
		</div>
	</div>
</form>


<?php
	}
}

/*
 * This function add in the WordPress menu the GUS panel
 */
function gus_admin_panel() 
{
	// if the user click on the GUS panel it launch the gus_admin_page function
	if (function_exists('add_management_page')) 
	       add_management_page('Gus Administration', 'GUS', 5, basename(__FILE__), 'gus_admin_page');  
}

/*
 * Uninstallation function of the plugin
 */     
function gus_uninstall() 
{ 
	// Delete the configuration
    delete_option('GUS_configuration');

    $gus_users = new WP_User_Query( array( 'meta_key' => 'gus_sync', 'meta_value' => 'Yes') );
    if ($gus_users->results)
    	foreach ($gus_users->results as $gus_user)
    		delete_user_meta($gus_user->ID,'gus_sync');
}

/*
 * Init function of the plugin
 */
function gus_init() {
	// get the directory where the plugin is installed
	$plugin_dir = basename(dirname(__FILE__));
	// Loads the plugin's translated strings. (ex: gus-plugin-fr_FR.mo)
	load_plugin_textdomain( 'gus-plugin', false, $plugin_dir );
}

/*
 * Log into galette without password
 */
function galette_login($user_login, $user) 
{
	$configuration=get_option('GUS_configuration');
	if (!$configuration['SSO_wp_gal']) return;
	global $zdb;
	global $i18n;
	global $login;
	global $session;
	include $configuration['galette_path'].'/includes/galette.inc.php';
	
	// Runkit must be installed to replace the password_verify function
	// http://us3.php.net/manual/en/runkit.installation.php
	// Redefine the password_verify to always respond true
	runkit_function_redefine('password_verify', '$password, $hash', 'return true;');

	// Log the user WordPress in Galette
	$login->logIn($user_login, '');

	// If user is logged in Galette send token
    if ( $login->isLogged() )
    	$session['login'] = serialize($login);	
}

/*
 * Log out galette
 */
function galette_logout()
{
	$configuration=get_option('GUS_configuration');
	if (!$configuration['SSO_wp_gal']) return;	
	global $zdb;
	global $i18n;
	global $login;
	global $session;
	include $configuration['galette_path'].'/includes/galette.inc.php';
		
    $login->logOut();
    $session['login'] = null;
    unset($session['login']);
}

// We add a new column in the WordPress User List
add_filter('manage_users_columns', 'gus_add_custom_field', 20, 1);
// We add the value of the new column in the WordPress User List
add_action('manage_users_custom_column', 'gus_custom_field_value', 20, 3);

// Here we as the GUS panel in the WordPress menu
add_action('admin_menu', 'gus_admin_panel');

// Launch the gus_uninstall function if the plugin is uninstalled
if ( function_exists('register_uninstall_hook') )
	register_uninstall_hook(__FILE__, 'gus_uninstall');

// When the plugin is loaded launch gus_init function
add_action('plugins_loaded', 'gus_init');

// Register the galette_login function when the user login for SSO with Galette
add_action('wp_login', 'galette_login', 10, 2);

// Register the galette_logout function when the user logout for SSO with Galette
add_action('wp_logout', 'galette_logout');


?>
